//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// AboutGUI class - creates a simple about-this-program dialog
//
import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Panel;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Button;
import java.awt.Label;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;

/**
 * Creates a simple about-this-program dialog.
 */
public class AboutGUI extends Dialog
		implements Runnable, ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// GUI components
	private ImageLabel	imagelabel;
	private Image		image_noanim;
	private Image		image_halfanim, image_fullanim;
	private Button		ok_button;
	private Frame		parent;

	// synchronization monitor, for setVisible notifications
	private Object		visible		= new Object();


	/**
	 * Initializes the whole GUI set.
	 */
	public AboutGUI( boolean inAnApplet, Frame frame, boolean flag ) {
		super( frame, flag );
		this.addWindowListener( this );
		parent		= frame;
		this.inAnApplet	= inAnApplet;

		// create GUI components
		setTitle( messages.getString( "AboutGUI.title" ));
		setLayout( new BorderLayout() );
		setForeground( Colour.find_colour(
			colours.getString( "AboutGUI.foreground" )));
		setBackground( Colour.find_colour(
			colours.getString( "AboutGUI.background" )));
		setFont( Font.decode(
			colours.getString( "AboutGUI.font" )));

		// create the image in the left of the GUI
		Panel image_panel = new Panel();
		image_panel.setLayout( new FlowLayout(
						FlowLayout.CENTER, 20, 20) );
		image_noanim	= createImage( Xpm.get_image_producer(
					messages.getString( "AboutGUI.xpm1" )));
		image_halfanim	= createImage( Xpm.get_image_producer(
					messages.getString( "AboutGUI.xpm2" )));
		image_fullanim	= createImage( Xpm.get_image_producer(
					messages.getString( "AboutGUI.xpm3" )));
		imagelabel	= new ImageLabel( image_noanim );
		image_panel.add( imagelabel );
		add( "West", image_panel );

		// create the labels, reading about_N properties until
		// no more are found
		Panel label_panel = new Panel();
		label_panel.setForeground( Colour.find_colour(
			colours.getString( "AboutGUI.panel_foreground" )));
		label_panel.setBackground( Colour.find_colour(
			colours.getString( "AboutGUI.panel_background" )));
		label_panel.setFont( Font.decode(
			colours.getString( "AboutGUI.panel_font" )));
		label_panel.setLayout( new GridLayout( 0, 1, 0, 0 ));
		for ( int index = 0; ; index++ ) {
			try {
				label_panel.add( new Label(
					messages.getString(
						"AboutGUI.about_" + index ),
								Label.CENTER ));
			}
			catch ( MissingResourceException e ) {
				break;
			}
		}
		add( "Center", label_panel );

		Panel button_panel = new Panel();
		button_panel.setLayout( new FlowLayout( FlowLayout.RIGHT ) );
		ok_button = new Button( messages.getString(
							"AboutGUI.okbutton" ));
		ok_button.addActionListener( this );
		button_panel.add ( ok_button );
		add( "South", button_panel );

		// start a new animation thread
		new Thread( this, "About_animation" ).start();
	}

	/**
	 * Thread to animate the image in the dialog.  For fun.
	 */
	public void run() {

		// sleep time in animations
		int sleep = 0;

		// begin with the normal image
		imagelabel.set_image( image_noanim );

		// loop forever, at the lowest priority
		Thread.currentThread().setPriority( Thread.MIN_PRIORITY );
		while ( true ) {

			// if our window isn't visible then just do nothing
			while ( ! isVisible() ) {
				try { synchronized( visible ) {
					visible.wait();
				}}
				catch ( InterruptedException e ) {}
			}

			// decide how long to sleep between animations
			if ( sleep > 5000 )
				sleep = (int)( Math.random() *  250f +  250f );
			else if ( sleep < 1000 )
				sleep = (int)( Math.random() * 4000f + 1000f );
			else
				sleep = (int)( Math.random() * 5000f + 1000f );

			// wait until its time to animate something
			try { Thread.sleep( sleep ); }
			catch ( InterruptedException e ) {}

			// animate the image
			imagelabel.set_image( image_halfanim );
			try { Thread.sleep( 50 ); }
			catch ( InterruptedException e ) {}
			imagelabel.set_image( image_fullanim );
			try { Thread.sleep( 50 ); }
			catch ( InterruptedException e ) {}
			imagelabel.set_image( image_halfanim );
			try { Thread.sleep( 50 ); }
			catch ( InterruptedException e ) {}
			imagelabel.set_image( image_noanim );
			try { Thread.sleep( 50 ); }
			catch ( InterruptedException e ) {}
		}
	}


	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_ok_button() {

		// simply just hide this frame from view
		setVisible( false );
	}

	/**
	 * Override setVisible to ensure we restart our thread when visible.
	 */
	public void setVisible( boolean is_visible ) {
		super.setVisible( is_visible );
		if ( is_visible )
			synchronized( visible ) { visible.notify(); }
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {
			if ( evt.getSource() == ok_button ) {
				handle_ok_button();
			}
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_ok_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowActivated( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowDeactivated( WindowEvent evt ) {}


	/*
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		Frame frame = new Frame();
		frame.setSize( 100, 100 );
		frame.setVisible( true );
		AboutGUI about = new AboutGUI( false, frame, false );
		about.pack(); about.setVisible( true );
	}
}
