//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// ExerciseGUIStats class - creates a dialog with lesson completion
// statistics displayed
//
import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Button;
import java.awt.Label;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;
import java.text.MessageFormat;

/**
 * A dialog for lesson completion statistics.
 */
public class ExerciseGUIStats extends Dialog
		implements ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// +/- limits on what we say is good rhythm, in %
	private static final int	RHYTHM_TOLERANCE	=
			Integer.parseInt( messages.getString(
					"ExerciseGUIStats.rhythmtolerance" ));
	// GUI components
	private Label		lesson_label;
	private Label		stats_line_1;
	private Label		stats_line_2;
	private Label		stats_line_3;
	private Button		ok_button;
	private Frame		parent;

	/**
	 * Initialize the whole GUI set.
	 */
	public ExerciseGUIStats( boolean inAnApplet, Frame frame,
			boolean flag ) {
		super( frame, flag );
		this.addWindowListener( this );
		parent		= frame;
		this.inAnApplet	= inAnApplet;

		// create GUI components
		setTitle( messages.getString( "ExerciseGUIStats.title" ));
		setLayout( new BorderLayout() );
		setForeground( Colour.find_colour(
			colours.getString( "ExerciseGUIStats.foreground" )));
		setBackground( Colour.find_colour(
			colours.getString( "ExerciseGUIStats.background" )));
		setFont( Font.decode(
			colours.getString( "ExerciseGUIStats.font" )));

		// create the image in the left of the GUI
		Panel image_panel = new Panel();
		image_panel.setLayout( new FlowLayout(
						FlowLayout.CENTER, 20, 20) );
		image_panel.add( new ImageLabel(
				Xpm.get_image_producer(
				messages.getString( "ExerciseGUIStats.xpm" ))));
		add( "West", image_panel );

		// create the labels
		Panel label_panel = new Panel();
		label_panel.setLayout( new GridLayout( 0, 1, 0, 0 ));
		label_panel.setForeground( Colour.find_colour(
			colours.getString( "ExerciseGUIStats.panel_foreground" )));
		label_panel.setBackground( Colour.find_colour(
			colours.getString( "ExerciseGUIStats.panel_background" )));
		label_panel.setFont( Font.decode(
			colours.getString( "ExerciseGUIStats.panel_font" )));
		lesson_label = new Label();
		label_panel.add( lesson_label );
		stats_line_1 = new Label();
		label_panel.add( stats_line_1 );
		stats_line_2 = new Label();
		label_panel.add( stats_line_2 );
		stats_line_3 = new Label();
		label_panel.add( stats_line_3 );
		add( "Center", label_panel );

		Panel button_panel = new Panel();
		button_panel.setLayout( new FlowLayout( FlowLayout.RIGHT ) );
		ok_button = new Button(
			messages.getString( "ExerciseGUIStats.okbutton" ));
		ok_button.addActionListener( this );
		button_panel.add ( ok_button );
		add( "South", button_panel );
	}


	/**
	 * Show a specific single set of statistics for a specific lesson.
	 */
	public void reveal( ExerciseLesson lesson,
			KeyingStatistics keying_stats,
			KeyingRhythm keying_rhythm ) {

		// update the labels with the details passed in
		Object[] fmtargs = null;
		fmtargs = new Object[] {
			lesson.get_title()
		};
		lesson_label.setText(
			MessageFormat.format(
			messages.getString( "ExerciseGUIStats.completed" ),
					fmtargs ));
		fmtargs = new Object[] {
			new Integer( keying_stats.get_strikes()
					+ keying_stats.get_misses() ),
			new Float( (float)keying_stats.get_pc_accuracy()/100f )
		};
		stats_line_1.setText(
			MessageFormat.format(
			messages.getString( "ExerciseGUIStats.line1" ),
					fmtargs ));
		fmtargs = new Object[] {
			new Integer( keying_stats.get_WPM( false ) ),
			new Integer( keying_stats.get_WPM( true ) )
		};
		stats_line_2.setText(
			MessageFormat.format(
			messages.getString( "ExerciseGUIStats.line2" ),
					fmtargs ));
		fmtargs = new Object[] {
			new Float( (float)keying_rhythm.get_pc_rhythm(
						RHYTHM_TOLERANCE )/100f )
		};
		stats_line_3.setText(
			MessageFormat.format(
			messages.getString( "ExerciseGUIStats.line3" ),
					fmtargs ));

		// call pack to resize the dialog box, then render
		pack(); setVisible( true );
	}


	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_ok_button() {

		// simply just hide this frame from view
		setVisible( false );
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {
			if ( evt.getSource() == ok_button ) {
				handle_ok_button();
			}
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_ok_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowActivated( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowDeactivated( WindowEvent evt ) {}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		Frame frame = new Frame();
		frame.setSize( 100, 100 );
		frame.setVisible( true );
		ExerciseGUIStats stats = new ExerciseGUIStats( false, frame,
				false );
		stats.pack();
		KeyingStatistics s = new KeyingStatistics();
		KeyingRhythm r = new KeyingRhythm();
		s.struck( 300 ); s.struck( 200 ); s.struck( 100 );
		s.missed( 400 );
		r.struck( 300 ); r.struck( 200 ); r.struck( 100 );
		r.struck( 400 );
		stats.reveal( new DrillLesson( "Test lesson", "", "" ), s, r );
	}
}
