//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// PopupGUI class - creates a simple popup dialog for messages
//
import java.awt.Frame;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Panel;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Button;
import java.awt.Label;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;
import java.text.MessageFormat;

/**
 * Creates a simple popup dialog for messages.
 */
public class PopupGUI extends Dialog
		implements ActionListener, WindowListener {
	private boolean inAnApplet = true;

	// message and colour catalog information
	private static final ResourceBundle	messages	=
				ResourceBundle.getBundle( "MessageResource" );
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// GUI components
	private Label		message_label;
	private Button		ok_button;
	private Frame		parent;

	/**
	 * Initialize the whole GUI set.
	 */
	public PopupGUI( boolean inAnApplet, Frame frame, boolean flag ) {
		super( frame, flag );
		this.addWindowListener( this );
		parent		= frame;
		this.inAnApplet	= inAnApplet;

		// create GUI components
		setTitle( messages.getString( "PopupGUI.title" ));
		setLayout( new BorderLayout() );
		setForeground( Colour.find_colour(
			colours.getString( "PopupGUI.foreground" )));
		setBackground( Colour.find_colour(
			colours.getString( "PopupGUI.background" )));
		setFont( Font.decode(
			colours.getString( "PopupGUI.font" )));

		// create the image in the left of the GUI
		Panel image_panel = new Panel();
		image_panel.setLayout( new FlowLayout(
						FlowLayout.CENTER, 20, 20) );
		image_panel.add( new ImageLabel(
				Xpm.get_image_producer(
					messages.getString( "PopupGUI.xpm" ))));
		add( "West", image_panel );

		// create the labels
		Panel label_panel = new Panel();
		label_panel.setForeground( Colour.find_colour(
			colours.getString( "PopupGUI.panel_foreground" )));
		label_panel.setBackground( Colour.find_colour(
			colours.getString( "PopupGUI.panel_background" )));
		label_panel.setFont( Font.decode(
			colours.getString( "PopupGUI.panel_font" )));
		label_panel.setLayout( new GridLayout( 0, 1, 0, 0 ));
		message_label = new Label( "", Label.CENTER );
		label_panel.add( message_label );
		add( "Center", label_panel );

		Panel button_panel = new Panel();
		button_panel.setLayout( new FlowLayout( FlowLayout.RIGHT ) );
		ok_button = new Button(
			messages.getString( "PopupGUI.okbutton" ));
		ok_button.addActionListener( this );
		button_panel.add ( ok_button );
		add( "South", button_panel );
	}


	/**
	 * Display the message text passed in.
	 */
	public void reveal( String message ) {

		// set up the dialog
		message_label.setText( message );
		pack(); setVisible( true );
	}


	/*
	 * Handle dismiss, close, etc by just hiding ourselves.
	 */
	private void handle_ok_button() {

		// simply just hide this frame from view
		setVisible( false );
	}


	/*
	 * Handle GUI component actions and events.
	 */
	public void actionPerformed( ActionEvent evt ) {
		if ( evt.getSource() instanceof Button ) {
			if ( evt.getSource() == ok_button ) {
				handle_ok_button();
			}
		}
	}
	public void windowClosing( WindowEvent evt ) {
		handle_ok_button();
	}
	public void windowClosed( WindowEvent evt ) {}
	public void windowOpened( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowIconified( WindowEvent evt ) {}
	public void windowDeiconified( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowActivated( WindowEvent evt ) {
		ok_button.requestFocus();
	}
	public void windowDeactivated( WindowEvent evt ) {}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		Frame frame = new Frame();
		frame.setSize( 100, 100 );
		frame.setVisible( true );
		PopupGUI popup = new PopupGUI( false, frame, false );
		popup.pack();
		popup.reveal( "Test popup message..." );
		try { Thread.sleep( 5000 ); }
		catch ( InterruptedException e ) {}
		popup.reveal( "Test popup message number 2..." );
	}
}
